package com.sample;

import org.apache.log4j.Level;
import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.SystemEventListenerFactory;
import org.drools.agent.KnowledgeAgent;
import org.drools.agent.KnowledgeAgentConfiguration;
import org.drools.agent.KnowledgeAgentFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderErrors;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.event.rule.DebugAgendaEventListener;
import org.drools.event.rule.DebugKnowledgeAgentEventListener;
import org.drools.event.rule.DebugWorkingMemoryEventListener;
import org.drools.io.ResourceChangeScannerConfiguration;
import org.drools.io.ResourceFactory;
import org.drools.logger.KnowledgeRuntimeLogger;
import org.drools.logger.KnowledgeRuntimeLoggerFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.rule.FactHandle;

/**
 * This is a sample class to launch a rule.
 */
public class DroolsTest {
	
	private KnowledgeAgentConfiguration agentConfig;
	private KnowledgeAgent kAgent;
	private StatefulKnowledgeSession ksession;

	public static final void main(String[] args) {
		SystemEventListenerFactory.setSystemEventListener(new DroolsSystemEventLogger());
    	DroolsTest droolsTest = new DroolsTest();

    	while(true) {
    	    try {
    	    	droolsTest.fire();
    	    	Thread.sleep(800);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
    	}
    }

	private void fire() {
		ksession.fireAllRules();
	}

	public DroolsTest() {
		super();
        try {
            // load up the knowledge base
            KnowledgeBase kbase = readKnowledgeBase();
            ksession = kbase.newStatefulKnowledgeSession();
            KnowledgeRuntimeLogger logger = KnowledgeRuntimeLoggerFactory.newThreadedFileLogger(ksession, "test", 2000);
            // go !
            // setup console debugging listeners
            ksession.addEventListener(new DebugAgendaEventListener());
            ksession.addEventListener(new DebugWorkingMemoryEventListener());

            Message message = new Message();
            message.setMessage("Hello World");
            message.setStatus(Message.HELLO);
            FactHandle fact = ksession.insert(message);
            
            Message message2 = new Message();
            message.setMessage("Hello World 2");
            message.setStatus(Message.HELLO);
            FactHandle fact2 = ksession.insert(message2);

            Message message3 = new Message();
            message.setMessage("Hello World 3");
            message.setStatus(Message.HELLO);
            FactHandle fact3 = ksession.insert(message3);
            

            //logger.close();
        } catch (Throwable t) {
            t.printStackTrace();
        }
         
	}
    
    private KnowledgeBase readKnowledgeBase() throws Exception {
        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
        // TODO Change Path to rule file!
        kbuilder.add(ResourceFactory.newFileResource("E:\\workspace\\TestDrools\\src\\main\\rules\\Sample.drl"), ResourceType.DRL);
        KnowledgeBuilderErrors errors = kbuilder.getErrors();
        if (errors.size() > 0) {
            for (KnowledgeBuilderError error: errors) {
                System.err.println(error);
            }
            throw new IllegalArgumentException("Could not parse knowledge.");
        }
        KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
        kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
        createKnowledgeAgent(kbase);

        return kAgent.getKnowledgeBase();
    }


	private void createKnowledgeAgent(KnowledgeBase kbase) {

		// first call, init all
		if(kAgent == null)
		{
	        agentConfig = KnowledgeAgentFactory.newKnowledgeAgentConfiguration();
			
	        // incremental changes to the knowlegde base on rule-file updates
	        agentConfig.setProperty("drools.agent.newInstance", "false");
	        // just to be safe. no directories, just files
	        agentConfig.setProperty( "drools.agent.scanDirectories", "false" ); 
	        
			// configurate the ResourceChangeServices
			ResourceChangeScannerConfiguration scannerConfig = ResourceFactory.getResourceChangeScannerService().newResourceChangeScannerConfiguration();
			scannerConfig.setProperty( "drools.resource.scanner.interval", "5" ); 
			ResourceFactory.getResourceChangeScannerService().configure(scannerConfig);

			// starts the resource polling and notification of it
			ResourceFactory.getResourceChangeNotifierService().start();
			ResourceFactory.getResourceChangeScannerService().start();
		}

		// remove stuff from old
        if(kAgent != null) {
    		// stop listening. important or else multiple updates will be executed because the old agents still work in the background
        	kAgent.monitorResourceChangeEvents(false);
        }

		// create agent
        kAgent = KnowledgeAgentFactory.newKnowledgeAgent("mainAgent", kbase, agentConfig);

        kAgent.addEventListener(new DebugKnowledgeAgentEventListener());		
	}

    public static class Message {

        public static final int HELLO = 0;
        public static final int GOODBYE = 1;

        private String message;

        private int status;

        public String getMessage() {
            return this.message;
        }

        public void setMessage(String message) {
            this.message = message;
        }

        public int getStatus() {
            return this.status;
        }

        public void setStatus(int status) {
            this.status = status;
        }

    }

}
